"""
    pyexcel_xlsbr
    ~~~~~~~~~~~~~~~~~~~
    The lower level xlsb file format handler
    :copyright: (c) 2019-2020 by Onni Software Ltd & its contributors
    :license: New BSD License
"""
# flake8: noqa
from pyexcel_io.io import get_data as read_data
from pyexcel_io.io import isstream
from pyexcel_io.plugins import IOPluginInfoChainV2

from ._version import __author__, __version__

__FILE_TYPE__ = "xlsb"

IOPluginInfoChainV2(__name__).add_a_reader(
    locations=["file"],
    relative_plugin_class_path="xlsbr.XLSBBook",
    file_types=[__FILE_TYPE__],
    stream_type="binary",
)


def get_data(afile, file_type=None, **keywords):
    """standalone module function for reading module supported file type"""
    if isstream(afile) and file_type is None:
        file_type = __FILE_TYPE__
    return read_data(afile, file_type=file_type, **keywords)



"""
    pyexcel_xlsbr.xlsbr
    ~~~~~~~~~~~~~~~~~~~
    The lower level xlsb file format handler
    :copyright: (c) 2018-2020 by Onni Software Ltd & its contributors
    :license: New BSD License
"""
from pyexcel_io.plugin_api import IReader, ISheet, NamedContent
from pyxlsb import open_workbook


class XLSBSheet(ISheet):
    def __init__(
        self,
        sheet,
        auto_detect_int=True,
        auto_detect_float=True,
        auto_detect_datetime=True,
    ):
        self.__auto_detect_int = auto_detect_int
        self.__auto_detect_float = auto_detect_float
        self.__auto_detect_datetime = auto_detect_datetime
        self._native_sheet = sheet

    def row_iterator(self):
        return self._native_sheet.rows()

    def column_iterator(self, row):
        for cell in row:
            yield self.__convert_cell(cell)

    def __convert_cell(self, cell):
        return cell.v


class XLSBBook(IReader):
    def __init__(self, file_name, file_type, **keywords):
        self._native_book = open_workbook(file_name)
        self._keywords = keywords
        self.content_array = []
        for sheet_index, sheet_name in enumerate(self._native_book.sheets, 1):
            sheet = self._native_book.get_sheet(sheet_index)
            self.content_array.append(NamedContent(sheet_name, sheet))

    def read_sheet(self, sheet_index):
        native_sheet = self.content_array[sheet_index].payload
        sheet = XLSBSheet(native_sheet, **self._keywords)
        return sheet

    def close(self):
        self._native_book.close()



import os

from nose.tools import eq_
from pyexcel_io._compact import OrderedDict

from pyexcel_xlsbr import get_data


def test_reading():
    data = get_data(
        os.path.join("tests", "fixtures", "date_field.xlsb"),
        library="pyexcel-xlsbr",
    )
    expected = OrderedDict()
    # expected.update({
    #    "Sheet1": [
    #        ['Date', 'Time'],
    #        [datetime(year=2014, month=12, day=25),
    #         time(hour=11, minute=11, second=11)],
    #        [datetime(2014, 12, 26, 0, 0),
    #         time(hour=12, minute=12, second=12)],
    #        [datetime(2015, 1, 1, 0, 0),
    #         time(hour=13, minute=13, second=13)],
    #        [datetime(year=1899, month=12, day=30),
    #         time(hour=0, minute=0)]
    #    ]})
    expected.update(
        {
            "Sheet1": [
                ["Date", "Time"],
                [41998.0, 0.466099537037037],
                [41999.0, 0.508472222222222],
                [42005.0, 0.550844907407407],
                [0.0, 0.0],
            ]
        }
    )
    expected.update({"Sheet2": []})
    expected.update({"Sheet3": []})
    eq_(data, expected)



# -*- coding: utf-8 -*-
DESCRIPTION = (
    'read Microsoft Excel tm xlsb file' +
    ''
)
# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))

# -- Project information -----------------------------------------------------

project = 'pyexcel-xlsbr'
copyright = '2015-2020 Onni Software Ltd.'
author = 'C.W.'
# The short X.Y version
version = '0.6.1'
# The full version, including alpha/beta/rc tags
release = '0.6.1'

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [    'sphinx.ext.autodoc',    'sphinx.ext.doctest',    'sphinx.ext.intersphinx',    'sphinx.ext.viewcode',]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'alabaster'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# -- Extension configuration -------------------------------------------------
# -- Options for intersphinx extension ---------------------------------------

# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {'https://docs.python.org/3/': None}
# TODO: html_theme not configurable upstream
html_theme = 'default'

# TODO: DESCRIPTION not configurable upstream
texinfo_documents = [
    ('index', 'pyexcel-xlsbr',
     'pyexcel-xlsbr Documentation',
     'Onni Software Ltd.', 'pyexcel-xlsbr',
     DESCRIPTION,
     'Miscellaneous'),
]
intersphinx_mapping.update({
    'pyexcel': ('http://pyexcel.readthedocs.io/en/latest/', None),
})
master_doc = "index"
